@@ -1,4 +1,4 @@
-import fsbc.SevenZipFile
+import fsbc.seven_zip_file
 import nose.tools
 
 
@@ -11,5 +11,5 @@ def test_mypy():
 
 def test_doctest():
     import doctest
-    failure_count, test_count = doctest.testmod(fsbc.SevenZipFile)
+    failure_count, test_count = doctest.testmod(fsbc.seven_zip_file)
     nose.tools.assert_equals(failure_count, 0)
