@@ -39,7 +39,6 @@
 
 #ifdef USE_OPENGL
 #include <fs/ml/opengl.h>
-#include <fs/glu.h>
 #endif
 
 #ifdef USE_GLES
@@ -1757,8 +1756,8 @@ static void render_fade_overlay(double alpha) {
 static void handle_quit_sequence() {
     int fade_time = fs_config_get_int_clamped("fade_out_duration", 0, 10000);
     if (fade_time == FS_CONFIG_NONE) {
-        // fade out over 750ms
-        fade_time = 750;
+        // fade out over 250ms
+        fade_time = 250;
     }
     fade_time = fade_time * 1000;
 
@@ -1814,7 +1813,7 @@ void fs_emu_video_render_function() {
     fs_emu_video_render_mutex_lock();
 
 
-    int in_menu = fs_emu_menu_is_active();
+    int in_menu = fs_emu_menu_mode();
     if (in_menu && g_menu_transition_target < 1.0) {
         g_menu_transition_target = 1.0;
     }
