@@ -1,3 +1,7 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #ifdef WINDOWS
 #define _WIN32_WINNT 0x0501
 #include <Windows.h>
@@ -17,12 +21,17 @@
 HICON hIconSmall = 0;
 HICON hIconBig = 0;
 
-void fs_ml_configure_window() {
+void fs_ml_configure_window(void)
+{
     fs_log("fs_ml_configure_window\n");
 
     SDL_SysWMinfo info;
     SDL_VERSION(&info.version); // this is important!
+#if SDL_VERSION_ATLEAST(2, 0, 0)
+    if (!SDL_GetWindowWMInfo(g_fs_ml_window, &info)) {
+#else
     if (!SDL_GetWMInfo(&info)) {
+#endif
         fs_log("error getting window information\n");
         return;
     }
@@ -38,18 +47,40 @@ void fs_ml_configure_window() {
         fs_log("hIconBig = %d\n", hIconBig);
     }
     if (hIconSmall != 0) {
+#if SDL_VERSION_ATLEAST(2, 0, 0)
+        SendMessage(info.info.win.window, WM_SETICON, ICON_SMALL, (LPARAM) hIconSmall);
+#else
         SendMessage(info.window, WM_SETICON, ICON_SMALL, (LPARAM) hIconSmall);
+#endif
     }
     if (hIconBig != 0) {
+#if SDL_VERSION_ATLEAST(2, 0, 0) 
+        SendMessage(info.info.win.window, WM_SETICON, ICON_BIG, (LPARAM) hIconBig);
+#else
         SendMessage(info.window, WM_SETICON, ICON_BIG, (LPARAM) hIconBig);
+#endif
+
     }
 }
 
-void fs_ml_prevent_power_saving(void) {
+void fs_ml_prevent_power_saving(void)
+{
+    // on Windows we only need to call a function once to change the thread
+    // state
+    static int initialized = 0;
+    if (initialized) {
+        return;
+    }
     fs_log("SetThreadExecutionState(ES_CONTINUOUS | ES_DISPLAY_REQUIRED)\n");
     if (!SetThreadExecutionState(ES_CONTINUOUS | ES_DISPLAY_REQUIRED)) {
         fs_log("SetThreadExecutionState failed\n");
     }
+    initialized = 1;
+}
+
+void fs_ml_activate_window_switcher_impl(void)
+{
+    fs_log("FIXME: Active window switcher not implemented\n");
 }
 
 // not used
@@ -92,10 +123,12 @@ void fs_ml_usleep(int usec) {
     SDL_Delay(usec / 1000);
 }
 
-void fs_ml_set_fullscreen_extra() {
-}
+//void fs_ml_set_fullscreen_extra() {
+//}
 
-int fs_ml_video_mode_get_current(fs_ml_video_mode *mode) {
+#if 0
+int fs_ml_video_mode_get_current(fs_ml_video_mode *mode)
+{
     DEVMODE devmode;
     devmode.dmSize = sizeof(DEVMODE);
     devmode.dmDriverExtra = 0;
@@ -110,6 +143,7 @@ int fs_ml_video_mode_get_current(fs_ml_video_mode *mode) {
     mode->flags = devmode.dmDisplayFlags;
     return 0;
 }
+#endif
 
 static STICKYKEYS g_StartupStickyKeys = {sizeof(STICKYKEYS), 0};
 static TOGGLEKEYS g_StartupToggleKeys = {sizeof(TOGGLEKEYS), 0};
@@ -158,7 +192,7 @@ void fs_ml_initialize_keymap() {
 #ifdef WITH_WINDOWS_VIDEO
 
 #include "video.h"
-#include <fs/glee.>
+// #include <fs/glee.>
 #include <Ddraw.h>
 
 /*
@@ -754,5 +788,9 @@ void fs_ml_input_init() {
 
 #endif // WITH_VIDEO_WINDOWS
 
+#else
+
+int libfsemu_ml_windows_dummy;
+
 #endif // WINDOWS
 
